package jalview.analysis.scoremodels;

import static org.testng.Assert.assertEquals;
import static org.testng.Assert.assertFalse;
import static org.testng.Assert.assertTrue;

import jalview.api.analysis.DistanceModelI;

import java.util.Iterator;

import org.testng.annotations.Test;

public class ScoreModelsTest
{
  /**
   * Verify that the singleton constructor successfully loads Jalview's built-in
   * score models
   */
  @Test
  public void testConstructor()
  {
    Iterator<DistanceModelI> models = ScoreModels.getInstance().getModels()
            .iterator();
    assertTrue(models.hasNext());

    /*
     * models are served in order of addition
     */
    DistanceModelI sm = models.next();
    assertTrue(sm instanceof PairwiseDistanceModel);
    assertEquals(sm.getName(), "BLOSUM62");
    assertEquals(((PairwiseDistanceModel) sm).getScoreModel()
            .getPairwiseScore('I', 'R'), -3f);

    sm = models.next();
    assertTrue(sm instanceof PairwiseDistanceModel);
    assertEquals(sm.getName(), "PAM250");
    assertEquals(((PairwiseDistanceModel) sm).getScoreModel()
            .getPairwiseScore('R', 'C'), -4f);

    sm = models.next();
    assertTrue(sm instanceof PairwiseDistanceModel);
    assertEquals(sm.getName(), "Identity (SeqSpace)");
    assertEquals(((PairwiseDistanceModel) sm).getScoreModel()
            .getPairwiseScore('R', 'C'), 0f);
    assertEquals(((PairwiseDistanceModel) sm).getScoreModel()
            .getPairwiseScore('R', 'r'), 1f);

    sm = models.next();
    assertTrue(sm instanceof PairwiseDistanceModel);
    assertEquals(sm.getName(), "DNA");
    assertEquals(((PairwiseDistanceModel) sm).getScoreModel()
            .getPairwiseScore('c', 'x'), 1f);

    sm = models.next();
    assertFalse(sm instanceof PairwiseDistanceModel);
    assertEquals(sm.getName(), "Sequence Feature Similarity");

    sm = models.next();
    assertFalse(sm instanceof PairwiseDistanceModel);
    assertEquals(sm.getName(), "PID");
  }
}
